@@ -4,19 +4,17 @@ from decimal import Decimal
 from simpleeval import simple_eval
 from itertools import chain
 
+from trytond.i18n import gettext
 from trytond.pool import Pool, PoolMeta
 from trytond.model import (
     ModelSQL, ModelView, Workflow, DeactivableMixin, fields)
 from trytond.pyson import Eval, Bool
 from trytond.tools import decistmt
 from trytond.transaction import Transaction
-from trytond.modules.company.model import CompanyMultiValueMixin, CompanyValueMixin
+from trytond.modules.company.model import (
+    CompanyMultiValueMixin, CompanyValueMixin)
 
-__all__ = [
-    'AdvancePaymentTerm', 'AdvancePaymentTermLine',
-    'AdvancePaymentTermLineAccount',
-    'AdvancePaymentCondition', 'Sale', 'SaleLine',
-    'HandleInvoiceException']
+from .exceptions import FormulaError
 
 
 class AdvancePaymentTerm(
@@ -58,7 +56,7 @@ class AdvancePaymentTermLine(ModelView, ModelSQL, CompanyMultiValueMixin):
     account = fields.MultiValue(
         fields.Many2One('account.account', "Account", required=True,
             domain=[
-                ('kind', '=', 'revenue'),
+                ('type.unearned_revenue', '=', True),
                 ],
             help="Used for the line of advance payment invoice."))
     accounts = fields.One2Many(
@@ -82,13 +80,10 @@ class AdvancePaymentTermLine(ModelView, ModelSQL, CompanyMultiValueMixin):
 
     @classmethod
     def __setup__(cls):
-        super(AdvancePaymentTermLine, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_formula': ('Invalid formula "%(formula)s" with'
-                    ' exception "%(exception)s".'),
-                })
+        super().__setup__()
+        cls.__access__.add('advance_payment_term')
 
-    @fields.depends('formula')
+    @fields.depends('formula', 'description')
     def pre_validate(self, **names):
         super(AdvancePaymentTermLine, self).pre_validate()
         names['total_amount'] = names['untaxed_amount'] = 0
@@ -96,10 +91,11 @@ class AdvancePaymentTermLine(ModelView, ModelSQL, CompanyMultiValueMixin):
             if not isinstance(self.compute_amount(**names), Decimal):
                 raise Exception('The formula does not return a Decimal')
         except Exception as exception:
-            self.raise_user_error('invalid_formula', {
-                    'formula': self.formula,
-                    'exception': exception,
-                    })
+            raise FormulaError(
+                gettext('sale_advance_payment.msg_term_line_invalid_formula',
+                    formula=self.formula,
+                    term_line=self.description or '',
+                    exception=exception)) from exception
 
     def get_compute_amount_context(self, **names):
         return {
@@ -136,6 +132,7 @@ class AdvancePaymentTermLineAccount(ModelSQL, CompanyValueMixin):
     account = fields.Many2One(
         'account.account', "Account", required=True,
         domain=[
+            ('type.unearned_revenue', '=', True),
             ('company', '=', Eval('company', -1)),
             ],
         depends=['company'])
@@ -167,7 +164,7 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
     account = fields.Many2One(
         'account.account', "Account", required=True,
         domain=[
-            ('kind', '=', 'revenue'),
+            ('type.unearned_revenue', '=', True),
             ('company', '=', Eval('sale_company')),
             ],
         states=_states,
@@ -222,9 +219,6 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
         return super(AdvancePaymentCondition, cls).copy(conditions, default)
 
     def create_invoice(self):
-        pool = Pool()
-        Invoice = pool.get('account.invoice')
-
         invoice = self.sale._get_invoice_sale()
         invoice.invoice_date = self.sale.sale_date
         if self.invoice_delay:
@@ -237,7 +231,7 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
         invoice.lines = invoice_lines
         invoice.save()
 
-        Invoice.update_taxes([invoice])
+        invoice.update_taxes()
         return invoice
 
     def get_invoice_advance_payment_lines(self, invoice):
@@ -251,7 +245,6 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
 
         invoice_line = InvoiceLine()
         invoice_line.invoice = invoice
-        invoice_line.company = invoice.company
         invoice_line.type = 'line'
         invoice_line.quantity = 1
         invoice_line.account = self.account
@@ -259,18 +252,19 @@ class AdvancePaymentCondition(ModelSQL, ModelView):
         invoice_line.description = self.description
         invoice_line.origin = self
         invoice_line.company = self.sale.company
+        invoice_line.currency = self.sale.currency
         # Set taxes
         invoice_line.on_change_account()
         return [invoice_line]
 
     def _get_advance_amount(self):
         return sum(l.amount for l in self.invoice_lines
-            if l.invoice.state != 'cancel')
+            if l.invoice.state != 'cancelled')
 
     def _get_ignored_amount(self):
         skips = {l for i in self.sale.invoices_recreated for l in i.lines}
         return sum(l.amount for l in self.invoice_lines
-            if l.invoice.state == 'cancel' and l not in skips)
+            if l.invoice.state == 'cancelled' and l not in skips)
 
     def get_completed(self, name):
         advance_amount = 0
@@ -298,8 +292,8 @@ class Sale(metaclass=PoolMeta):
             'readonly': Eval('state') != 'draft',
             },
         depends=['state'])
-    advance_payment_invoices = fields.Function(fields.One2Many(
-            'account.invoice', None, "Advance Payment Invoices"),
+    advance_payment_invoices = fields.Function(fields.Many2Many(
+            'account.invoice', None, None, "Advance Payment Invoices"),
         'get_advance_payment_invoices',
         searcher='search_advance_payment_invoices')
 
@@ -362,7 +356,7 @@ class Sale(metaclass=PoolMeta):
         invoices = [i
             for i in self.advance_payment_invoices if i.id not in skip_ids]
         if invoices:
-            if any(i.state == 'cancel' for i in invoices):
+            if any(i.state == 'cancelled' for i in invoices):
                 return 'exception'
             elif all(i.state == 'paid' for i in invoices):
                 return state
@@ -398,7 +392,6 @@ class Sale(metaclass=PoolMeta):
 
     def create_invoice(self):
         pool = Pool()
-        Invoice = pool.get('account.invoice')
         InvoiceLine = pool.get('account.invoice.line')
 
         invoice = super(Sale, self).create_invoice()
@@ -413,7 +406,7 @@ class Sale(metaclass=PoolMeta):
                     for line in recall_lines:
                         line.invoice = invoice
                     InvoiceLine.save(recall_lines)
-                    Invoice.update_taxes([invoice])
+                    invoice.update_taxes()
 
         return invoice
 
@@ -481,16 +474,13 @@ class HandleInvoiceException(metaclass=PoolMeta):
     __name__ = 'sale.handle.invoice.exception'
 
     def default_ask(self, fields):
-        pool = Pool()
-        Sale = pool.get('sale.sale')
-
         default = super(HandleInvoiceException, self).default_ask(fields)
         invoices = default['domain_invoices']
 
-        sale = Sale(Transaction().context['active_id'])
+        sale = self.record
         skips = set(sale.invoices_ignored)
         skips.update(sale.invoices_recreated)
         for invoice in sale.advance_payment_invoices:
-            if invoice.state == 'cancel' and invoice not in skips:
+            if invoice.state == 'cancelled' and invoice not in skips:
                 invoices.append(invoice.id)
         return default
